@@ -1,7 +1,7 @@
 # This file is part of pybliographer
 # 
-# Copyright (C) 1998,1999,2000 Frederic GOBRY
-# Email : gobry@idiap.ch
+# Copyright (C) 1998-2004 Frederic GOBRY
+# Email : gobry@pybliographer.org
 # 	   
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: BibTeX.py,v 1.21.2.5 2001/12/05 13:39:53 fredgo Exp $
+# 
 
 # Extension module for BibTeX files
 
@@ -29,10 +29,6 @@ from Pyblio.Fields import *
 from Pyblio import Base, Config, Autoload, Types
 from Pyblio import Open, Key, Utils, Iterator, Exceptions
 
-import gettext
-_ = gettext.gettext
-
-
 # this database is shared between all the unpickled entries
 _unpickle_db = _bibtex.open_string ("<unpickled>", '', 0);
 
@@ -61,7 +57,7 @@ def _fieldtype (field):
     return 0
 
 
-extended_date = re.compile ('^[\{"]\s*(\d+)\s*[\}"]\s*#\s*(\w+)$')
+extended_date = re.compile ('^[\{"]?\s*(\d+)\s*[\}"]?\s*#\s*(\w+)$')
 
 
 class BibTextField (Text):
@@ -421,11 +417,11 @@ class DataBase (Base.DataBase):
 	       (self.key, len (self))
 
 
-    def get_native (self, key):
+    def get_native (self, entry):
 	''' Return the object in its native format '''
 
 	stream = Utils.StringStream ()
-	entry_write (self.dict [key], stream)
+	entry_write (entry, stream)
 
 	return stream.text
 
@@ -482,6 +478,8 @@ def entry_write (entry, output):
     for key in convert.keys ():
         monthlist [convert [key] - 1] = key
 
+    dateformat = Config.get ('bibtex+/dateformat').data
+    
     if native:
 	# loop over all the fields
 	for field in entry.keys ():
@@ -494,7 +492,8 @@ def entry_write (entry, output):
 		if date.month:
                     month = monthlist [date.month - 1]
                     if date.day:
-                        month = '{%d } # %s' % (date.day, month)
+                        month = dateformat % {'day':    date.day,
+                                              'month' : month}
                         
 		    dico [datefields [field] [1]] = month
 			 
@@ -514,7 +513,8 @@ def entry_write (entry, output):
 		if date.month:
                     month = monthlist [date.month - 1]
                     if date.day:
-                        month = '{%d } # %s' % (date.day, month)
+                        month = dateformat % {'day':    date.day,
+                                              'month' : month}
                         
 		    dico [datefields [field] [1]] = month
 			 
@@ -531,6 +531,9 @@ def entry_write (entry, output):
 
                 dico [field] = _nativify (value, fieldtype)
 
+
+    first = True
+    
     # write according to the type order
     for f in tp.mandatory + tp.optional:
 
@@ -538,17 +541,26 @@ def entry_write (entry, output):
 	field = string.lower (f.name)
 	if not dico.has_key (field): continue
 
+        if not first: output.write (',\n')
+        else:         first = False
+            
 	output.write ('  %-14s = ' % f.name)
         output.write (Utils.format (dico [field],
-                                    75, 19, 19) [19:] + ',\n')
+                                    75, 19, 19) [19:])
 	del dico [field]
 
-    for f in dico.keys ():
+    keys = dico.keys ()
+    keys.sort ()
+    
+    for f in keys:
+        if not first: output.write (',\n')
+        else:         first = False
+            
 	output.write ('  %-14s = ' % f)
 	output.write (Utils.format (dico [f],
-				   75, 19, 19) [19:] + ',\n')
+				   75, 19, 19) [19:])
 
-    output.write ('}\n\n')
+    output.write ('\n}\n\n')
     return
 
 
