@@ -1,7 +1,8 @@
+# -*- coding: latin-1 -*-
 # This file is part of pybliographer
 # 
-# Copyright (C) 1998,1999,2000 Frederic GOBRY
-# Email : gobry@idiap.ch
+# Copyright (C) 1998-2004 Frederic GOBRY
+# Email : gobry@pybliographer.org
 # 	   
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,7 +18,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Medline.py,v 1.5.2.5 2002/01/06 00:27:14 fredgo Exp $
+# 
 
 # Extension module for Medline files
 
@@ -30,6 +31,8 @@ contin = re.compile ('^      (.*)$')
 
 one_to_one = Config.get ('medline/mapping').data
 
+medurl = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/elink.fcgi?cmd=prlinks&dbfrom=pubmed&retmode=ref&id='
+
 
 class MedlineIterator (Iterator.Iterator):
 
@@ -90,6 +93,11 @@ class MedlineIterator (Iterator.Iterator):
         # create the entry with the actual fields
         norm = {}
         type = Types.get_entry ('article')
+
+        if table.has_key ('PMID'):
+            norm ['url'] = Fields.URL (medurl + table ['PMID'] [0])
+            norm ['medline-pmid'] = Fields.Text (table ['PMID'] [0])
+            del table ['PMID']
     
         if table.has_key ('UI'):
             norm [one_to_one ['UI']] = Fields.Text (table ['UI'] [0])
