@@ -1,7 +1,7 @@
 # This file is part of pybliographer
 # 
-# Copyright (C) 1998,1999,2000 Frederic GOBRY
-# Email : gobry@idiap.ch
+# Copyright (C) 1998-2004 Frederic GOBRY
+# Email : gobry@pybliographer.org
 # 	   
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,16 +17,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Editor.py,v 1.18.2.5 2002/01/13 16:14:27 fredgo Exp $
+# 
+
+# TO FIX
+#   entry editor for more than 50 chars
 
 import string, re
-from gnome.ui import *
-from gnome import config
-from gtk import *
-import GTK
+from gnome import ui
+import gtk
 
-import gettext, copy, re
-_ = gettext.gettext
+import copy, re
 
 from Pyblio import Fields, Config, Base, Types, Connector, Exceptions, Key
 
@@ -39,7 +39,7 @@ _newcontent = {
     Fields.Text        : _("Text"),
     Fields.URL         : 'http://',
     Fields.Date        : '2000',
-    Fields.Reference   : _("Reference"),
+    Fields.Reference   : [],
     }
 
 
@@ -48,10 +48,10 @@ class BaseField (Connector.Publisher):
     
     def __init__ (self, entry, field, content, j):
 
-        self.w = GtkVBox ()
+        self.w = gtk.VBox ()
 
-        h = GtkHBox (spacing = 5)
-        self.w.pack_start (GtkLabel (field), FALSE, FALSE)
+        h = gtk.HBox (spacing = 5)
+        self.w.pack_start (gtk.Label (field), False, False)
 
         field = string.lower (field)
         self.field = field
@@ -61,31 +61,32 @@ class BaseField (Connector.Publisher):
         self.edit = None
         expand = self.create_widget (h)
         
-        if self.edit:
-            self.edit.connect ('key_press_event', self.key_handler)
+        img = gtk.Image ()
+        
+        if self.loss: img.set_from_stock (gtk.STOCK_CANCEL,
+                                          gtk.ICON_SIZE_BUTTON)
+        else:         img.set_from_stock (gtk.STOCK_APPLY,
+                                          gtk.ICON_SIZE_BUTTON)
+        
+        h.pack_start (img, False, False)
         
-        if self.loss: h.pack_start (GnomeStock (STOCK_BUTTON_NO),
-                                    FALSE, FALSE)
-        else:         h.pack_start (GnomeStock (STOCK_BUTTON_YES),
-                                    FALSE, FALSE)
-
         self.w.pack_start (h, expand, expand)
         self.w.show_all ()
 
         flag = 0
-        if expand: flag = EXPAND | FILL
+        if expand: flag = gtk.EXPAND | gtk.FILL
         content.attach (self.w, 0, 1, j, j + 1, yoptions = flag)
         return
 
 
     def key_handler (self, widget, ev):
-        if ev.keyval == GDK.Return and \
-           ev.state  == GDK.CONTROL_MASK:
+        if ev.keyval == gtk.gdk.Return and \
+           ev.state  == gtk.gdk.CONTROL_MASK:
             widget.emit_stop_by_name ('key_press_event')
             self.issue ('apply')
         
-        elif ev.keyval == GDK.Tab and \
-           ev.state  == GDK.CONTROL_MASK:
+        elif ev.keyval == gtk.gdk.Tab and \
+           ev.state  == gtk.gdk.CONTROL_MASK:
             widget.emit_stop_by_name ('key_press_event')
             self.issue ('next')
 
@@ -103,7 +104,7 @@ class BaseField (Connector.Publisher):
 
     
     def update (self, entry):
-        text = string.strip (self.edit.get_chars (0, -1))
+        text = string.strip (self.edit.get_chars (0, -1)).encode ('latin-1')
 
         if text == self.string: return 0
 
@@ -126,8 +127,135 @@ class TextBase (BaseField):
         return
 
     
+    def update_content (self, entry, text):
+        if text [0] == '@' and hasattr (entry, 'set_native'):
+            try:
+                entry.set_native (self.field, string.lstrip (text [1:]))
+                
+            except Exceptions.ParserError, msg:
+                Utils.error_dialog (_("Error in native string parsing"), str (msg))
+                return -1
+            
+            return 1
+
+        text = string.lstrip (text)
+        entry [self.field] = Fields.Text (text)
+        return 1
+
+    
+class Entry (TextBase):
+
+    def create_widget (self, h):
+        if len (self.string) < 50:
+            self.edit = gtk.Entry ()
+            self.edit.set_text (self.string.decode ('latin-1'))
+            self.edit.set_editable (True)
+            self.edit.show ()
+
+            self.buff = None
+            
+            h.pack_start (self.edit)
+            return 0
+
+        w = gtk.ScrolledWindow ()
+        w.set_policy (gtk.POLICY_NEVER,
+                      gtk.POLICY_AUTOMATIC)
+        
+        self.edit = gtk.TextView ()
+        self.edit.set_editable (True)
+        self.edit.set_wrap_mode (gtk.WRAP_WORD)
+
+        self.buff = self.edit.get_buffer ()
+        self.buff.set_text (self.string.decode ('latin-1'))
+
+        self.edit.show ()
+        
+        w.add (self.edit)
+        w.show ()
+        
+        h.pack_start (w)
+        return 1
+
+
     def update (self, entry):
-        text = string.rstrip (self.edit.get_chars (0, -1))
+        if self.buff:
+            text = self.buff.get_text (self.buff.get_start_iter (),
+                                       self.buff.get_end_iter ())
+            text = string.rstrip (text).encode ('latin-1')
+        else:
+            text = string.rstrip \
+                   (self.edit.get_chars (0, -1)).encode ('latin-1')
+            
+        if text == self.string: return 0
+
+        if text == '':
+            del entry [self.field]
+            return 1
+
+        return self.update_content (entry, text)
+
+
+
+class Text (TextBase):
+    
+    def create_widget (self, h):
+        w = gtk.ScrolledWindow ()
+        w.set_policy (gtk.POLICY_NEVER,
+                      gtk.POLICY_AUTOMATIC)
+        self.edit = gtk.TextView ()
+        self.edit.set_editable (True)
+        self.edit.set_wrap_mode (gtk.WRAP_WORD)
+
+        self.buff = self.edit.get_buffer ()
+        self.buff.set_text (self.string.decode ('latin-1'))
+
+        self.edit.show ()
+        w.add (self.edit)
+        w.show ()
+
+        h.pack_start (w)
+        return 1
+
+
+    def update (self, entry):
+        text = self.buff.get_text (self.buff.get_start_iter (),
+                                   self.buff.get_end_iter ())
+        text = string.rstrip (text).encode ('latin-1')
+        
+        if text == self.string: return 0
+
+        if text == '':
+            del entry [self.field]
+            return 1
+
+        return self.update_content (entry, text)
+
+
+class AuthorGroup (BaseField):
+    
+    def create_widget (self, h):
+        w = gtk.ScrolledWindow ()
+        w.set_policy (gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        
+        self.edit = gtk.TextView ()
+        self.edit.set_wrap_mode (gtk.WRAP_WORD)
+        self.edit.set_editable (True)
+
+        self.buff = self.edit.get_buffer ()
+        self.buff.set_text (self.string.decode ('latin-1'))
+        self.edit.show ()
+        w.add (self.edit)
+        w.show ()
+
+        h.pack_start (w)
+        return 1
+
+
+    def update (self, entry):
+        text = self.buff.get_text (self.buff.get_start_iter (),
+                                   self.buff.get_end_iter ())
+        text = string.strip (text).encode ('latin-1')
+        
         if text == self.string: return 0
 
         if text == '':
@@ -138,77 +266,6 @@ class TextBase (BaseField):
         return 1
 
 
-    def update_content (self, entry, text):
-        if text [0] == '@' and hasattr (entry, 'set_native'):
-            entry.set_native (self.field, string.lstrip (text [1:]))
-            return
-
-        text = string.lstrip (text)
-        entry [self.field] = Fields.Text (text)
-        return
-
-    
-class Entry (TextBase):
-
-    def create_widget (self, h):
-        if len (self.string) < 50:
-            self.edit = GtkEntry ()
-            self.edit.set_text (self.string)
-            self.edit.set_editable (TRUE)
-            self.edit.show ()
-            
-            h.pack_start (self.edit)
-            return 0
-
-        w = GtkScrolledWindow ()
-        w.set_policy (GTK.POLICY_NEVER, GTK.POLICY_AUTOMATIC)
-        self.edit = GtkText ()
-        self.edit.set_editable (TRUE)
-        self.edit.insert_defaults (self.string)
-        self.edit.set_word_wrap (TRUE)
-        self.edit.show ()
-        
-        w.add (self.edit)
-        w.show ()
-        
-        h.pack_start (w)
-        return 1
-
-
-class Text (TextBase):
-    
-    def create_widget (self, h):
-        w = GtkScrolledWindow ()
-        w.set_policy (GTK.POLICY_NEVER, GTK.POLICY_AUTOMATIC)
-        self.edit = GtkText ()
-        self.edit.set_editable (TRUE)
-        self.edit.insert_defaults (self.string)
-        self.edit.set_word_wrap (TRUE)
-        self.edit.show ()
-        w.add (self.edit)
-        w.show ()
-
-        h.pack_start (w)
-        return 1
-
-
-class AuthorGroup (BaseField):
-    
-    def create_widget (self, h):
-        w = GtkScrolledWindow ()
-        w.set_policy (GTK.POLICY_NEVER, GTK.POLICY_AUTOMATIC)
-        self.edit = GtkText ()
-        self.edit.set_editable (TRUE)
-        self.edit.set_word_wrap (TRUE)
-        self.edit.insert_defaults (self.string)
-        self.edit.show ()
-        w.add (self.edit)
-        w.show ()
-
-        h.pack_start (w)
-        return 1
-
-
     def setup (self, entry):
         if entry.has_key (self.field):
             (self.value, self.loss)  = entry.field_and_loss (self.field)
@@ -245,35 +302,35 @@ class AuthorGroup (BaseField):
 class Date (BaseField):
     
     def create_widget (self, h):
-        hbox = GtkHBox (FALSE, 5)
+        hbox = gtk.HBox (False, 5)
 
-        self.day = GtkEntry ()
+        self.day = gtk.Entry ()
         (width, height) = self.day.size_request ()
-        self.day.set_usize (width / 4, height)
+        self.day.set_size_request (width / 4, height)
         self.day.set_max_length (2)
-        self.day.connect ('key_press_event', self.key_handler)
+
         if self.initial [0]:
-            self.day.set_text (str (self.initial [0]))
+            self.day.set_text (str (self.initial [0]).decode ('latin-1'))
         hbox.pack_start (self.day)
-        hbox.pack_start (GtkLabel (_("Day")), FALSE, FALSE)
+        hbox.pack_start (gtk.Label (_("Day")), False, False)
         
-        self.month = GtkEntry ()
-        self.month.set_usize (width / 4, height)
+        self.month = gtk.Entry ()
+        self.month.set_size_request (width / 4, height)
         self.month.set_max_length (2)
-        self.month.connect ('key_press_event', self.key_handler)
+
         if self.initial [1]:
-            self.month.set_text (str (self.initial [1]))
+            self.month.set_text (str (self.initial [1]).decode ('latin-1'))
         hbox.pack_start (self.month)
-        hbox.pack_start (GtkLabel (_("Month")), FALSE, FALSE)
+        hbox.pack_start (gtk.Label (_("Month")), False, False)
         
-        self.year = GtkEntry ()
+        self.year = gtk.Entry ()
         self.year.set_max_length (4)
-        self.year.set_usize (width / 3, height)
-        self.year.connect ('key_press_event', self.key_handler)
+        self.year.set_size_request (width / 3, height)
+
         if self.initial [2]:
-            self.year.set_text (str (self.initial [2]))
+            self.year.set_text (str (self.initial [2]).decode ('latin-1'))
         hbox.pack_start (self.year)
-        hbox.pack_start (GtkLabel (_("Year")), FALSE, FALSE)
+        hbox.pack_start (gtk.Label (_("Year")), False, False)
 
         hbox.show_all ()
         h.pack_start (hbox)
@@ -294,28 +351,28 @@ class Date (BaseField):
     def update (self, entry):
         (day, month, year) = (None, None, None)
         
-        text = string.strip (self.day.get_chars (0, -1))
+        text = string.strip (self.day.get_chars (0, -1)).encode ('latin-1')
         if text != '':
             try: day = int (text)
             except ValueError:
-                GnomeErrorDialog (_("Invalid day field in date"),
-                                  self.day.get_toplevel ()).show ()
+                ui.gnome_error_dialog_parented (_("Invalid day field in date"),
+                                                self.day.get_toplevel ())
                 return -1
         
-        text = string.strip (self.month.get_chars (0, -1))
+        text = string.strip (self.month.get_chars (0, -1)).encode ('latin-1')
         if text != '':
             try: month = int (text)
             except ValueError, err:
-                GnomeErrorDialog (_("Invalid month field in date"),
-                                  self.day.get_toplevel ()).show ()
+                ui.gnome_error_dialog_parented (_("Invalid month field in date"),
+                                                self.day.get_toplevel ())
                 return -1
         
-        text = string.strip (self.year.get_chars (0, -1))
+        text = string.strip (self.year.get_chars (0, -1)).encode ('latin-1')
         if text != '':
             try: year = int (text)
             except ValueError: 
-                GnomeErrorDialog (_("Invalid year field in date"),
-                                  self.day.get_toplevel ()).show ()
+                ui.gnome_error_dialog_parented (_("Invalid year field in date"),
+                                                self.day.get_toplevel ())
                 return -1
         
         if self.initial == (day, month, year): return 0
@@ -327,8 +384,8 @@ class Date (BaseField):
         try:
             entry [self.field] = Fields.Date ((year, month, day))
         except Exceptions.DateError, error:
-            GnomeErrorDialog (str (error),
-                              self.day.get_toplevel ()).show ()
+            ui.gnome_error_dialog_parented (str (error),
+                                            self.day.get_toplevel ())
             return -1
         return 1
 
@@ -337,31 +394,61 @@ class Reference (BaseField):
     
     def create_widget (self, h):
         accept = (
-            (Mime.KEY_TYPE,   0, Mime.KEY),
+            (Mime.SYM_KEY, 0, Mime.KEY),
             )
 
-        self.edit = GtkEntry ()
-        self.edit.set_editable (FALSE)
-        self.edit.set_text (self.string)
-        self.edit.show ()
-        self.edit.drag_dest_set (DEST_DEFAULT_MOTION |
-                                 DEST_DEFAULT_HIGHLIGHT |
-                                 DEST_DEFAULT_DROP,
-                                 accept,
-                                 GDK.ACTION_COPY)
-        self.edit.connect ('drag_data_received', self.drag_received)
-        self.current = None
+        box = gtk.HBox ()
+        box.set_border_width (5)
         
-        h.pack_start (self.edit)
+        self.edit = gtk.Label ()
+        self.edit.justify = False
+
+        self.edit.set_line_wrap (True)
+        self.edit.set_selectable (True)
+        
+        #self.edit.set_editable (True)
+        self.edit.set_text (self.string)
+        
+        self.edit.drag_dest_set (gtk.DEST_DEFAULT_ALL,
+                                 accept,
+                                 gtk.gdk.ACTION_COPY)
+        
+        self.edit.connect ('drag_data_received', self.drag_received)
+        
+        box.pack_start (self.edit, True, True)
+        h.pack_start (box)
+
+        # A delete button
+        button = gtk.Button (_('Delete'), gtk.STOCK_DELETE)
+        button.connect ('clicked', self._delete)
+        
+        h.pack_start (button, False, False)
+        
+        box.show_all ()
         return 0
 
+    def _delete (self, * args):
+
+        self.current = Fields.Reference ([])
+        self.string  = _("[ Drop an Entry here ]")
+
+        self.edit.set_text (self.string)
+        return
+    
+
     def setup (self, entry):
         if entry.has_key (self.field):
-            (self.value, self.loss)  = entry.field_and_loss (self.field)
-            self.string = string.join (map (lambda x: x.key, self.value.list), ', ')
+            (self.current, self.loss)  = entry.field_and_loss (self.field)
+            
+            if self.current.list:
+                self.string = string.join (map (lambda x: x.key, self.current.list), ', ')
+            else:
+                self.string = _("[ Drop an Entry here ]")
         else:
-            (self.value, self.loss) = (None, 0)
-            self.string = ''
+            (self.current, self.loss) = (Fields.Reference ([]), 0)
+            self.string = _("[ Drop an Entry here ]")
+
+        self.initial = self.current
         return
     
     def drag_received (self, *arg):
@@ -385,17 +472,25 @@ class Reference (BaseField):
         return
     
 
-    def update_content (self, entry, text):
+    def update (self, entry):
+
+        if self.current == Fields.Reference ([]):
+            del entry [self.field]
+            return 1
+
+        if self.current == self.initial: return 0
+
         entry [self.field] = self.current
-        return
+        return 1
+
 
 
 class URL (BaseField):
     
     def create_widget (self, h):
-        self.edit = GtkEntry ()
-        self.edit.set_editable (TRUE)
-        self.edit.set_text (self.string)
+        self.edit = gtk.Entry ()
+        self.edit.set_editable (True)
+        self.edit.set_text (self.string.decode ('latin-1'))
         self.edit.show ()
 
         h.pack_start (self.edit)
@@ -418,19 +513,18 @@ class RealEditor (Connector.Publisher):
 
         self.fields.sort ()
         
-        self.w = GtkVBox ()
-        table  = GtkTable (2, 2)
+        self.w = gtk.VBox ()
+        table  = gtk.Table (2, 2)
         table.set_border_width (5)
         table.set_col_spacings (5)
         
-        table.attach (GtkLabel (_("Entry type")),
+        table.attach (gtk.Label (_("Entry type")),
                       0, 1, 0, 1, yoptions = 0)
-        table.attach (GtkLabel (_("Key")),
+        table.attach (gtk.Label (_("Key")),
                       1, 2, 0, 1, yoptions = 0)
 
-        self.key = GtkEntry ()
-        self.key.set_editable (TRUE)
-        self.key.connect ('key_press_event', self.key_handler)
+        self.key = gtk.Entry ()
+        self.key.set_editable (True)
         
         if self.entry.key:
             self.key.set_text (self.entry.key.key)
@@ -438,8 +532,8 @@ class RealEditor (Connector.Publisher):
         table.attach (self.key,
                       1, 2, 1, 2, yoptions = 0)
 
-        self.menu = GtkOptionMenu ()
-        menu = GtkMenu ()
+        self.menu = gtk.OptionMenu ()
+        menu = gtk.Menu ()
         self.menu.set_menu (menu)
 
         table.attach (self.menu,
@@ -459,26 +553,25 @@ class RealEditor (Connector.Publisher):
         self.menu.set_history (history)
         
         table.show_all ()
-        self.w.pack_start (table, FALSE, FALSE)
+        self.w.pack_start (table, False, False)
 
-        self.newfield_area = GtkHBox (spacing = 5)
+        self.newfield_area = gtk.HBox (spacing = 5)
         self.newfield_area.set_border_width (5)
-        self.newfield = GnomeEntry ('newfield')
-        self.newfield.load_history ()
+        self.newfield = ui.Entry ('newfield')
         
         self.newfield_area.pack_start (self.newfield)
 
-        b = GtkButton (_("Create Field"))
+        b = gtk.Button (_("Create Field"))
         b.connect ('clicked', self.create_field)
         self.newfield_area.pack_start (b)
         self.newfield_area.show_all ()
         
-        self.w.pack_start (self.newfield_area, FALSE, FALSE)
+        self.w.pack_start (self.newfield_area, False, False)
         
         # Notebook
-        # scroll = GtkScrolledWindow ()
+        # scroll = gtk.ScrolledWindow ()
         # scroll.set_policy (GTK.POLICY_AUTOMATIC, GTK.POLICY_AUTOMATIC)
-        self.notebook = GtkNotebook ()
+        self.notebook = gtk.Notebook ()
         self.notebook.show ()
 
         #scroll.add_with_viewport (self.notebook)
@@ -486,7 +579,7 @@ class RealEditor (Connector.Publisher):
         self.w.pack_start (self.notebook)
         self.w.show_all ()
         
-        self.notebook_init = FALSE
+        self.notebook_init = False
         self.update_notebook ()
         return
 
@@ -507,20 +600,6 @@ class RealEditor (Connector.Publisher):
         return
 
 
-    def key_handler (self, widget, ev):
-        if ev.keyval == GDK.Return and \
-           ev.state  == GDK.CONTROL_MASK:
-            widget.emit_stop_by_name ('key_press_event')
-            self.issue ('apply')
-        
-        elif ev.keyval == GDK.Tab and \
-           ev.state  == GDK.CONTROL_MASK:
-            widget.emit_stop_by_name ('key_press_event')
-            self.issue ('next')
-
-        return 1
-
-
     def apply_cb (self, * arg):
         self.issue ('apply')
         return
@@ -535,14 +614,14 @@ class RealEditor (Connector.Publisher):
             for i in range (0, 3):
                 self.notebook.remove_page (0)
         
-        self.notebook_init = TRUE
+        self.notebook_init = True
 
         names  = (_("Mandatory"), _("Optional"), _("Extra"))
         fields = map (string.lower, self.entry.keys ())
         
         self.content = []
         for i in range (0, 3):
-            label   = GtkLabel (names [i])
+            label   = gtk.Label (names [i])
 
             if   i == 0: table = map (lambda x: x.name, self.entry.type.mandatory)
             elif i == 1: table = map (lambda x: x.name, self.entry.type.optional)
@@ -550,10 +629,11 @@ class RealEditor (Connector.Publisher):
 
             if len (table) == 0: continue
 
-            scroll = GtkScrolledWindow ()
-            scroll.set_policy (GTK.POLICY_AUTOMATIC, GTK.POLICY_AUTOMATIC)
+            scroll = gtk.ScrolledWindow ()
+            scroll.set_policy (gtk.POLICY_AUTOMATIC,
+                               gtk.POLICY_AUTOMATIC)
 
-            content = GtkTable (1, len (table))
+            content = gtk.Table (1, len (table))
             
             scroll.add_with_viewport (content)
 
@@ -597,33 +677,42 @@ class RealEditor (Connector.Publisher):
     
 
     def update (self, database, entry):
-        modified = FALSE
+        modified = False
         
         key = string.strip (self.key.get_text ())
         if key == '':
             self.entry.key = None
-            modified = TRUE
+            modified = True
         else:
             if not key_re.match (key):
-                GnomeErrorDialog (_("Invalid key format"),
-                                  self.w.get_toplevel ())
+                ui.gnome_error_dialog_parented (_("Invalid key format"),
+                                                self.w.get_toplevel ())
                 return None
 
             key = Key.Key (database, key)
 
             if key != self.entry.key:
                 if database.has_key (key):
-                    GnomeErrorDialog (_("Key `%s' already exists") % str (key.key),
-                                      self.w.get_toplevel ())
+                    ui.gnome_error_dialog_parented (_("Key `%s' already exists") % str (key.key),
+                                                    self.w.get_toplevel ())
                     return None
                 
                 self.entry.key = key
-                modified = TRUE
+                modified = True
                 
         modified = self.type != self.entry.type or modified
         
         for item in self.content:
-            result = item.update (self.entry)
+            try:
+                result = item.update (self.entry)
+                
+            except UnicodeEncodeError:
+                f = Types.get_field (item.field)
+                
+                ui.gnome_error_dialog_parented (_("The `%s' field contains a non Latin-1 symbol") %
+                                                f.name, self.w.get_toplevel ())
+                return None
+            
             if result == -1: return None
             
             modified = result or modified
@@ -646,51 +735,65 @@ class NativeEditor (Connector.Publisher):
 
         self.entry    = entry
         self.database = database
+
         if database.has_key (entry.key):
-            self.original = database.get_native (entry.key)
+            self.original = database.get_native (entry)
         else:
             self.original = ''
+
+        self.w = gtk.ScrolledWindow ()
+        self.w.set_policy (gtk.POLICY_NEVER,
+                           gtk.POLICY_AUTOMATIC)
         
-        self.w = GtkText ()
-        self.w.set_editable (TRUE)
-        self.w.connect ('key_press_event', self.key_handler)
+        self.w_txt = gtk.TextView ()
+        self.w_txt.set_editable (True)
+        self.w_txt.set_wrap_mode (gtk.WRAP_WORD)
         
-        self.w.insert (Config.get ('gnomeui/monospaced').data,
-                       None, None, self.original)
+        self.w.add (self.w_txt)
+
+        self.w_txt.show ()
+        
+        self.buff = self.w_txt.get_buffer ()
+
+        iter = self.buff.get_start_iter ()
+        mono = self.buff.create_tag ('body', family = 'Monospace')
+
+        self.buff.insert_with_tags (iter, self.original.decode ('latin-1'), mono)
         return
 
 
-    def key_handler (self, widget, ev):
-        if ev.keyval == GDK.Return and \
-           ev.state  == GDK.CONTROL_MASK:
-            widget.emit_stop_by_name ('key_press_event')
-            self.issue ('apply')
-        
-        elif ev.keyval == GDK.Tab and \
-           ev.state  == GDK.CONTROL_MASK:
-            widget.emit_stop_by_name ('key_press_event')
-            self.issue ('next')
-
-        return 1
-
-
     def update (self, database, entry):
         ''' updates and returns the new entry '''
+
         new  = None
+
+        text = self.buff.get_text (self.buff.get_start_iter (),
+                                   self.buff.get_end_iter ())
+
         try:
-            new = self.database.create_native (self.w.get_chars (0, -1))
-        except Exceptions.ParserError, error:
+            text = text.encode ('latin-1')
+            
+        except UnicodeEncodeError:
+            ui.gnome_error_dialog_parented (_("Your text contains non Latin-1 symbols"),
+                                            self.w.get_toplevel ())
+            return None
+        
+
+        try:
+            new = self.database.create_native (text)
+            
+        except Exceptions.ParserError, msg:
             Utils.error_dialog (_("Error in native string parsing"),
-                                str (error))
+                                str (msg))
         return new
 
     
 class Editor (Connector.Publisher):
     
     def __init__ (self, database, entry, parent = None, title = None):
-        self.w = GtkDialog ()
+        self.w = gtk.Dialog ()
         
-        self.w.set_policy (TRUE, TRUE, FALSE)
+        self.w.set_resizable (True)
         
         if title: self.w.set_title (title)
         else:     self.w.set_title (_("Edit entry") + ' [%s]' % str (entry.key) )
@@ -699,7 +802,7 @@ class Editor (Connector.Publisher):
 
         if parent: self.w.set_transient_for (parent)
 
-        self.apply_b = GnomeStockButton (STOCK_BUTTON_APPLY)
+        self.apply_b = gtk.Button (stock = gtk.STOCK_APPLY)
         self.apply_b.connect ('clicked', self.apply_changes)
         self.apply_b.show ()
 
@@ -707,21 +810,22 @@ class Editor (Connector.Publisher):
         self.has_native = hasattr (database, 'get_native')
 
         if self.has_native:
-            self.native_b = GtkButton (_("Native Editing"))
+            self.native_b = gtk.Button (_("Native Editing"))
             self.native_b.connect ('clicked', self.toggle_native)
             self.native_b.show ()
         
-        self.close_b = GnomeStockButton (STOCK_BUTTON_CANCEL)
+        self.close_b = gtk.Button (stock = gtk.STOCK_CANCEL)
         self.close_b.connect ('clicked', self.close_dialog)
         self.close_b.show ()
 
         # Use Escape to abort, Ctrl-Return to accept
-        accelerator = GtkAccelGroup ()
+        accelerator = gtk.AccelGroup ()
         self.w.add_accel_group (accelerator)
 
-        self.close_b.add_accelerator ('clicked', accelerator, GDK.Escape, 0, 0)
-        self.apply_b.add_accelerator ('clicked', accelerator, GDK.Return, GDK.CONTROL_MASK,
-                                 0)
+        self.close_b.add_accelerator ('clicked', accelerator,
+                                      gtk.keysyms.Escape, 0, 0)
+        self.apply_b.add_accelerator ('clicked', accelerator,
+                                      gtk.keysyms.Return, gtk.gdk.CONTROL_MASK, 0)
 
         self.w.action_area.add (self.apply_b)
         if self.has_native: self.w.action_area.add (self.native_b)
@@ -731,6 +835,9 @@ class Editor (Connector.Publisher):
         self.database    = database
         self.editor      = None
 
+        # this is the working copy of the entry
+        self.current     = copy.deepcopy (entry)
+        
         # put the negated value, so that we can call toggle to switch and create
         self.native_mode = not (self.has_native and
                                 Config.get ('gnome/native-as-default').data)
@@ -745,29 +852,48 @@ class Editor (Connector.Publisher):
         self.save_size ()
         
         if self.native_mode:
+
+            if self.editor:
+                cur = self.editor.update (self.database, self.current)
+
+                # Reject the switch if the data is invalid
+                if cur is None: return
+                
+                self.editor.w.destroy ()
+                self.current = cur
+                
             # real edition
-            self.native_mode = FALSE
-            if self.has_native:
-                self.native_b.children () [0].set_text (_("Native Editing"))
-
-            if self.editor: self.editor.w.destroy ()
-            self.editor = RealEditor (self.database,
-                                      copy.deepcopy (self.entry))
+            self.native_mode = False
             
-            ui_width  = config.get_int ('Pybliographic/Editor/Width=-1')
-            ui_height = config.get_int ('Pybliographic/Editor/Height=-1')
+            if self.has_native:
+                self.native_b.get_children () [0].set_text (_("Native Editing"))
+
+            
+            self.editor = RealEditor (self.database, self.current)
+            
+            ui_width  = Utils.config.get_int ('/apps/pybliographic/editor/width')  or -1
+            ui_height = Utils.config.get_int ('/apps/pybliographic/editor/height') or -1
+
         else:
-            # native edition
-            self.native_mode = TRUE
-            if self.has_native:
-                self.native_b.children () [0].set_text (_("Standard Editing"))
-            
-            if self.editor: self.editor.w.destroy ()
-            self.editor = NativeEditor (self.database,
-                                        copy.deepcopy (self.entry))
+            if self.editor:
+                cur = self.editor.update (self.database, self.current)
 
-            ui_width  = config.get_int ('Pybliographic/Native/Width=-1')
-            ui_height = config.get_int ('Pybliographic/Native/Height=-1')
+                # Reject the switch if the data is invalid
+                if cur is None: return
+                
+                self.editor.w.destroy ()
+                self.current = cur
+
+            # native edition
+            self.native_mode = True
+
+            if self.has_native:
+                self.native_b.get_children () [0].set_text (_("Standard Editing"))
+            
+            self.editor = NativeEditor (self.database, self.current)
+
+            ui_width  = Utils.config.get_int ('/apps/pybliographic/native/width')  or -1
+            ui_height = Utils.config.get_int ('/apps/pybliographic/native/height') or -1
 
 
         self.editor.Subscribe ('apply', self.apply_changes)
@@ -777,8 +903,9 @@ class Editor (Connector.Publisher):
 
         # set window size
         if ui_width != -1 and ui_height != -1:
-            self.editor.w.set_usize(ui_width, ui_height)
-            
+            self.w.set_default_size (ui_width, ui_height)
+            self.w.resize (ui_width, ui_height)
+        
         self.editor.w.show ()
         return
 
@@ -789,20 +916,19 @@ class Editor (Connector.Publisher):
         n = self.editor.notebook
         box = n.get_nth_page (n.get_current_page ())
         
-        box.focus (GTK.DIR_DOWN)
+        box.focus (DIR_DOWN)
         pass
 
 
     def save_size (self):
         if not self.editor: return
         
-        alloc = self.editor.w.get_allocation ()
-        if self.native_mode: field = 'Native'
-        else:                field = 'Editor'
-        
-        config.set_int ('Pybliographic/%s/Width' % field,  alloc [2])
-        config.set_int ('Pybliographic/%s/Height' % field, alloc [3])
-        config.sync ()
+        w, h = self.w.get_size ()
+        if self.native_mode: field = 'native'
+        else:                field = 'editor'
+
+        Utils.config.set_int ('/apps/pybliographic/%s/width' % field,  w)
+        Utils.config.set_int ('/apps/pybliographic/%s/height' % field, h)
         return
 
     
@@ -813,7 +939,7 @@ class Editor (Connector.Publisher):
 
 
     def apply_changes (self, * arg):
-        new = self.editor.update (self.database, self.entry)
+        new = self.editor.update (self.database, self.current)
         if new:
             self.close_dialog ()
             if new is not self.entry:
