@@ -1,7 +1,7 @@
 # This file is part of pybliographer
 # 
-# Copyright (C) 1998,1999,2000 Frederic GOBRY
-# Email : gobry@idiap.ch
+# Copyright (C) 1998-2004 Frederic GOBRY
+# Email : gobry@pybliographer.org
 # 	   
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,18 +17,23 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Index.py,v 1.11.2.6 2001/10/24 16:56:55 fredgo Exp $
+# 
+
+
+# TO FIX
+#
+#  - DnD with the world
+#  - Copy/Paste with the world
+
 
 ''' Main index containing the columned view of the entries '''
 
-from Pyblio import Fields, Config, Connector, Types
+from Pyblio import Fields, Config, Connector, Types, Sort
 
-from gnome.ui import *
-from gnome import config
+from gnome import ui
+import gtk, gobject
 
 from Pyblio.GnomeUI import FieldsInfo, Utils, Mime
-from gtk import *
-import GTK, GDK
 
 from string import *
 
@@ -45,85 +50,106 @@ class Index (Connector.Publisher):
     
     def __init__ (self, fields = None):
         ''' Creates a new list of entries '''
-
+        
         fields = fields or Config.get ('gnome/columns').data
         self.fields = map (lower, fields)
 
-        self.clist = GtkCList (len (fields), map (lambda x: ' ' + x + ' ', fields))
-        self.clist.set_selection_mode (GTK.SELECTION_EXTENDED)
+        self.model = apply (gtk.ListStore,
+                            (gobject.TYPE_STRING,) * len (fields))
+
+        self.list = gtk.TreeView ()
+        self.list.set_model (self.model)
         
-        self.w = GtkScrolledWindow ()
-        self.w.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
-        self.w.add (self.clist)
+        self.selinfo = self.list.get_selection ()
+        self.selinfo.set_mode (gtk.SELECTION_MULTIPLE)
+        
+        i = 0
+        for f in fields:
+            col = gtk.TreeViewColumn (f, gtk.CellRendererText (),
+                                      text = i)
+            col.set_resizable (True)
+            col.set_clickable (True)
+
+            k = '/apps/pybliographic/columns/%s' % self.fields [i]
+            w = Utils.config.get_int (k)
+
+            if w:
+                col.set_sizing (gtk.TREE_VIEW_COLUMN_FIXED)
+                col.set_fixed_width (w)
+            
+            col.connect ('clicked', self.click_column, i)
+            
+            self.list.append_column (col)
+            i = i + 1
+        
+        self.w = gtk.ScrolledWindow ()
+
+        self.w.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.w.add (self.list)
 
         self.access = []
 
-        self.menu_position = {
-            'add'    : 0,
-            'edit'   : 1,
-            'delete' : 2,
-            }
+        # contextual popup menu
+        self.menu = gtk.Menu ()
         
-        self.menu = GnomePopupMenu ([
-            UIINFO_ITEM_STOCK(_("Add..."), None, self.entry_new,    STOCK_MENU_NEW),
-            UIINFO_ITEM      (_("Edit..."),None, self.entry_edit),
-            UIINFO_ITEM_STOCK(_("Delete"), None, self.entry_delete, STOCK_MENU_TRASH),
-            ])
+        Utils.popup_add (self.menu, _('Add...'),
+                         self.entry_new)
+        Utils.popup_add (self.menu, _('Edit...'),
+                         self.entry_edit)
+        Utils.popup_add (self.menu, _('Delete...'),
+                         self.entry_delete)
         
-        self.menu.show ()
-
-        self.field_width = []
-        # resize the columns
-        for c in range (len (self.fields)):
-            width = config.get_int ('Pybliographic/Columns/%s=-1' % self.fields [c])
-            if width == -1:
-                width = FieldsInfo.width (self.fields [c])
-
-            self.field_width.append (width)
-            
-            self.clist.set_column_width (c, width)
-            self.clist.set_column_justification (c, FieldsInfo.justification (self.fields [c]))
-            
         # some events we want to react to...
-        self.clist.connect ('click_column',       self.click_column)
-        self.clist.connect ('resize_column',      self.resize_column)
-        self.clist.connect ('select_row',         self.select_row)
-        self.clist.connect ('button_press_event', self.button_press)
+        self.selinfo.connect ('changed', self.select_row)
 
-        # ---------- DnD configuration
+        self.list.connect ('row-activated', self.entry_edit)
+        self.list.connect ('button-press-event', self.button_press)
 
+        # DnD configuration
+ 
         targets = (
-            (Mime.KEY_TYPE,   0, Mime.KEY),
-            (Mime.ENTRY_TYPE, 0, Mime.ENTRY),
+            (Mime.SYM_KEY,    0, Mime.KEY),
+            (Mime.SYM_ENTRY,  0, Mime.ENTRY),
+            (Mime.SYM_STRING, 0, Mime.STRING),
+            (Mime.SYM_TEXT  , 0, Mime.STRING),
             )
-
+ 
         accept = (
-            (Mime.ENTRY_TYPE, 0, Mime.ENTRY),
+            (Mime.SYM_ENTRY,  0, Mime.ENTRY),
             )
-
-        self.clist.drag_dest_set (DEST_DEFAULT_MOTION |
-                                  DEST_DEFAULT_HIGHLIGHT |
-                                  DEST_DEFAULT_DROP,
-                                  accept,
-                                  GDK.ACTION_COPY | GDK.ACTION_MOVE)
-        self.clist.connect ("drag_data_received", self.drag_received)
-
-
-        self.clist.drag_source_set (GDK.BUTTON1_MASK | GDK.BUTTON3_MASK,
-                                    targets, GDK.ACTION_COPY | GDK.ACTION_MOVE)
-        self.clist.connect ('drag_data_get', self.dnd_drag_data_get)
-
-        # ---------- Copy/Paste configuration
-
+ 
+        self.list.drag_dest_set (gtk.DEST_DEFAULT_ALL, accept,
+                                 gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
+        self.list.connect ("drag_data_received", self.drag_received)
+ 
+ 
+        self.list.drag_source_set (gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
+                                   targets, gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
+        self.list.connect ('drag_data_get', self.dnd_drag_data_get)
+ 
+        # Copy/Paste configuration
+ 
         self.selection_buffer = None
-        
-        self.clist.connect ('selection_received', self.selection_received)
-        self.clist.connect ('selection_get', self.selection_get)
-        self.clist.connect ('selection_clear_event', self.selection_clear)
+         
+        self.list.connect ('selection_received', self.selection_received)
+        self.list.connect ('selection_get', self.selection_get)
+        self.list.connect ('selection_clear_event', self.selection_clear)
 
-        self.clist.selection_add_target (1, Mime.atom ['STRING'], 0)
-        self.clist.selection_add_target (1, Mime.atom [Mime.ENTRY_TYPE],
-                                         Mime.ENTRY)
+        # We handle three selections: one specific to the application,
+        # the clipboard, and the primary. Therefore, we should be able
+        # to paste into every kind of editor/application.
+        
+        self.list.selection_add_target ("PRIMARY",
+                                        Mime.SYM_STRING,
+                                        Mime.STRING)
+        
+        self.list.selection_add_target ("CLIPBOARD",
+                                        Mime.SYM_STRING,
+                                        Mime.STRING)
+        
+        self.list.selection_add_target (Mime.SYM_APP,
+                                        Mime.SYM_ENTRY,
+                                        Mime.ENTRY)
         return
 
 
@@ -133,19 +159,27 @@ class Index (Connector.Publisher):
 
     
     def selection_received (self, widget, selection, info):
-        if selection.length < 0: return
+        
+        data = selection.data
+        
+        if not data: return
  
-        entries = pickle.loads (selection.data)
+        entries = pickle.loads (data)
         self.issue ('drag-received', entries)
         return
 
 
     def selection_get (self, widget, selection, info, time):
+
         if not self.selection_buffer: return
-        
+
         if info == Mime.ENTRY:
+            
             text = pickle.dumps (self.selection_buffer)
-        else:
+            selection.set (Mime.SYM_ENTRY, 8, text)
+
+        elif info == Mime.STRING:
+            
             if Config.get ('gnome/paste-key').data:
                 # if we want the keys, return the keys !
                 keys = []
@@ -155,29 +189,28 @@ class Index (Connector.Publisher):
             else:
                 # else, return the full entries
                 text = join (map (str, self.selection_buffer), '\n\n')
+
+            selection.set ("STRING", 8, text)
         
-        selection.set (1, 8, text)
         return
 
 
     def selection_copy (self, entries):
-        self.clist.selection_owner_set (1, 0)
+        # Advertise the fact that we hold the selection
+        self.list.selection_owner_set (Mime.SYM_APP)
+        self.list.selection_owner_set ("PRIMARY")
+        self.list.selection_owner_set ("CLIPBOARD")
+        
         self.selection_buffer = entries
         return
 
 
     def selection_paste (self):
-        self.clist.selection_convert (1, Mime.atom [Mime.ENTRY_TYPE], 0)
+        # Request the selection as a full entry
+        self.list.selection_convert (Mime.SYM_APP, Mime.SYM_ENTRY)
         return
 
         
-    def set_menu_active (self, item, value):
-        ''' This method sets the sensitivity of each popup menu item '''
-
-        self.menu.children () [self.menu_position [item]].set_sensitive (value)
-        return
-    
-
     def drag_received (self, * arg):
         selection = arg [4]
         info      = arg [5]
@@ -194,6 +227,20 @@ class Index (Connector.Publisher):
 
         entries = self.selection ()
         if not entries: return
+
+        if info == Mime.STRING:
+            if Config.get ('gnome/paste-key').data:
+                # if we want the keys, return the keys !
+                keys = []
+                for e in entries:
+                    keys.append (str (e.key.key))
+                text = join (keys, ',')
+            else:
+                # else, return the full entries
+                text = join (map (str, entries), '\n\n')
+                
+            selection.set (selection.target, 8, text)
+            return
         
         if info == Mime.KEY:
             # must return a set of keys
@@ -205,7 +252,10 @@ class Index (Connector.Publisher):
             data = pickle.dumps (entries)
             selection.set (selection.target, 8, data)
 
-        if context.action == GDK.ACTION_MOVE:
+        else:
+            return
+        
+        if context.action == gtk.gdk.ACTION_MOVE:
             self.issue ('drag-moved', entries)
         
         return
@@ -229,10 +279,12 @@ class Index (Connector.Publisher):
         if type (item) is not type (1):
             item = self.get_item_position (item)
 
-        if item == -1: return
+        if item == -1 or item >= len (self.access): return
+
+        path = (item,)
         
-        self.clist.select_row (item, 0)
-        self.set_scroll (item)
+        self.selinfo.select_path (path)
+        self.list.scroll_to_cell (path)
 
         self.issue ('select-entry', self.access [item])
         return
@@ -243,8 +295,10 @@ class Index (Connector.Publisher):
             item = self.get_item_position (item)
 
         if item == -1: return
-            
-        self.clist.moveto (item, 0, .5, 0)
+        
+        self.list.scroll_to_cell ((item,),
+                                  use_align = True,
+                                  row_align = .5)
         return
 
     
@@ -255,14 +309,17 @@ class Index (Connector.Publisher):
 
         Utils.set_cursor (self.w, 'clock')
         
-        self.clist.freeze ()
-        self.clist.clear ()
+        self.model.clear ()
 
         entry = iterator.first ()
         while entry:
             row = []
-            
+
+            i = 0
             for f in self.fields:
+                row.append (i)
+                i = i + 1
+                
                 if f == '-key-':
                     row.append (str (entry.key.key))
                     
@@ -276,51 +333,85 @@ class Index (Connector.Publisher):
                     else:
                         text = str (entry [f])
                         
-                    row.append (text)
+                    row.append (text.decode ('latin-1'))
                 else:
                     row.append ('')
 
-            self.clist.append  (row)
+            iter = self.model.append  ()
+            apply (self.model.set, [iter] + row)
+            
             self.access.append (entry)
 
             entry = iterator.next ()
             
-        self.clist.thaw ()
         Utils.set_cursor (self.w, 'normal')
         return
 
+
+    def go_to_first (self, query, field):
+        ''' Go to the first entry that matches a key '''
+        if not isinstance (field, Sort.FieldSort): return 0
+
+        f = field.field
+        q = lower (query)
+        l = len (q)
+        i = 0
+        
+        for e in self.access:
+            if not e.has_key (f): continue
+
+            c = cmp (lower (str (e [f])) [0:l], q)
+
+            if c == 0:
+                # matching !
+                self.set_scroll (i)
+                return 1
+
+            if c > 0:
+                # we must be after the entry...
+                self.set_scroll (i)
+                return 0
+            
+            i = i + 1
+
+        # well, show the user its entry must be after the bounds
+        self.set_scroll (i)
+        return  0
+        
     
-    def click_column (self, clist, column, * data):
+    def click_column (self, listcol, column):
         ''' handler for column title selection '''
         
         self.issue ('click-on-field', self.fields [column])
         return
 
 
-    def resize_column (self, clist, column, width):
-        self.field_width [column] = width
-        return
-
-    
-    def select_row (self, clist, row, column, * data):
+    def select_row (self, sel, * data):
         ''' handler for row selection '''
+
+        entries = self.selection ()
         
-        # multiple selections are not handled like single selections
-        if len (self.clist.selection) > 1:
-            self.issue ('select-entries',
-                        map (lambda x, self=self: self.access [x],
-                             self.clist.selection))
+        if len (entries) > 1:
+            self.issue ('select-entries', entries)
+            return
+
+        if len (entries) == 1:
+            self.issue ('select-entry', entries [0])
             return
-        
-        self.issue ('select-entry', self.access [row])
         return
 
 
     def selection (self):
         ''' returns the current selection '''
         
-        return map (lambda x, self=self: self.access [x],
-                    self.clist.selection)
+        entries = []
+
+        def retrieve (model, path, iter):
+            entries.append (self.access [path [0]])
+
+        self.selinfo.selected_foreach (retrieve)
+
+        return entries
 
 
     def select_all (self):
@@ -333,65 +424,49 @@ class Index (Connector.Publisher):
     def button_press (self, clist, event, *arg):
         ''' handler for double-click and right mouse button '''
 
-        if (event.type == GDK._2BUTTON_PRESS and event.button == 1):
-            # select the item below the cursor
-            couple = self.clist.get_selection_info (event.x, event.y)
-            if couple:
-                self.clist.select_row (couple [0], couple [1])
-                self.issue ('edit-entry', [self.access [couple [0]]])
-            return
+        if not (event.type == gtk.gdk.BUTTON_PRESS and
+                event.button == 3): return
         
-        if (event.type == GDK.BUTTON_PRESS and event.button == 3):
-            if not self.clist.selection:
-                # select the item below the cursor
-                couple = self.clist.get_selection_info (event.x, event.y)
-                if couple:
-                    self.clist.select_row (couple [0], couple [1])
-
-            # what menu items are accessible ?
-
-            if len (self.clist.selection) == 0:
-                mask = (1, 0, 0)
-            else:
-                mask = (1, 1, 1)
-
-            items = ('add', 'edit', 'delete')
-            for i in range (3):
-                self.set_menu_active (items [i], mask [i])
-
-            self.menu.popup (None, None, None, event.button, event.time)
-            return
+        # what menu items are accessible ?
+        sel = self.selection ()
+            
+        if len (sel) == 0: mask = (1, 0, 0)
+        else:              mask = (1, 1, 1)
 
+        child = self.menu.get_children ()
+        for i in range (3):
+            child [i].set_sensitive (mask [i])
+            
+        self.menu.popup (None, None, None, event.button, event.time)
         return
 
 
     def entry_new (self, * arg):
-        self.set_menu_active ('add', 0)
-        
-        self.issue ('new-entry', map (lambda x, self=self: self.access [x],
-                                       self.clist.selection))
+        self.issue ('new-entry')
         return
 
     
     def entry_edit (self, * arg):
-        if not self.clist.selection: return
+        sel = self.selection ()
+        if not sel: return
 
-        self.issue ('edit-entry', map (lambda x, self=self: self.access [x],
-                                       self.clist.selection))
+        self.issue ('edit-entry', sel)
         return
 
         
     def entry_delete (self, * arg):
-        if not self.clist.selection: return
+        sel = self.selection ()
+        if not sel: return
         
-        self.issue ('delete-entry', map (lambda x, self=self: self.access [x],
-                                       self.clist.selection))
+        self.issue ('delete-entry', sel)
         return
 
 
     def update_configuration (self):
 
         for i in range (len (self.fields)):
-            config.set_int ('Pybliographic/Columns/%s' % self.fields [i],
-                            self.field_width [i])
+            w = self.list.get_column (i).get_width ()
+            k = '/apps/pybliographic/columns/%s' % self.fields [i]
+            
+            Utils.config.set_int (k, w)
         return
