@@ -1,9 +1,6 @@
 # Site configuration
 
-from Pyblio.Types  import *
-from Pyblio.Fields import *
-
-from Pyblio import Autoload, Config
+from Pyblio import Autoload, Config, version
 
 from Pyblio.TextUI import *
 
@@ -17,35 +14,30 @@ Autoload.preregister ('format', 'BibTeX',  'Pyblio.Format.BibTeX',  '.*\.bib')
 Autoload.preregister ('format', 'Ovid',    'Pyblio.Format.Ovid',    '.*\.ovid')
 Autoload.preregister ('format', 'Medline', 'Pyblio.Format.Medline', '.*\.med')
 Autoload.preregister ('format', 'Refer',   'Pyblio.Format.Refer',   '.*\.refer')
+Autoload.preregister ('format', 'ISIFile', 'Pyblio.Format.isifile', '.*\.isi')
 
 
 # define styles and outputs
 
-Autoload.preregister ('style', 'Alpha',  'Pyblio.Style.alpha')
-Autoload.preregister ('style', 'Abbrv',  'Pyblio.Style.alpha')
-Autoload.preregister ('style', 'Custom', 'Pyblio.Style.custom')
+Autoload.preregister ('style', 'Generic', 'Pyblio.Style.Generic')
+Autoload.preregister ('style', 'apa4e',   'Pyblio.Style.apa4e')
 
 Autoload.preregister ('output', 'Text',  'Pyblio.Output.text')
 Autoload.preregister ('output', 'Raw',   'Pyblio.Output.raw')
 Autoload.preregister ('output', 'HTML',  'Pyblio.Output.html')
 Autoload.preregister ('output', 'LaTeX', 'Pyblio.Output.LaTeX')
 
+# define key formats
+
+Autoload.preregister ('key', 'Default', 'Pyblio.Utils')
 
 # Parse the configuration directory
 
 rootconfig = os.path.join ('Pyblio', 'ConfDir')
 
 if not os.path.isdir (rootconfig):
-    rootconfig = os.path.join (pyb_prefix, 'Pyblio', 'ConfDir')
+    rootconfig = os.path.join (version.pybdir, 'Pyblio', 'ConfDir')
     
 if os.path.isdir (rootconfig):
     Config.parse_directory (rootconfig)
 
-
-# ==================================================
-
-
-# ==================================================
-
-
-    
