@@ -1,9 +1,8 @@
-#! @bin_dir@/pybliographer -q
-
+# -*- python -*-
 # This file is part of pybliographer
 # 
-# Copyright (C) 1998 Frederic GOBRY
-# Email : gobry@idiap.ch
+# Copyright (C) 1998-2004 Frederic GOBRY
+# Email : gobry@pybliographer.org
 # 	   
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,21 +18,28 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: pybliographic.py,v 1.5 1999/09/02 12:57:50 gobry Exp $
+# 
 
+import sys
 
-from Pyblio.GnomeUI import Pybliographic
-from gtk import *
-from Pyblio import Base, Config
+from Pyblio.GnomeUI.Pybliographic import Pybliographic
+from Pyblio.Fields import URL
 
-import os
+from Pyblio.GnomeUI import ui_version
 
-main = Pybliographic.Pybliographic (version)
-main.show ()
+print ui_version
 
-if len (sys.argv) > 1:
-    main.open (os.path.abspath (sys.argv [1]))
+main = Pybliographic ()
 
-mainloop ()
+if len (sys.argv) > 2:
+    for file in sys.argv [2:]:
+        url = URL (file)
+        
+        main.open_document (str (url))
+else:
+    main.new_document ()
 
+import gtk
+
+gtk.main ()
 
